<?php

/**
 * @license GPLv3, http://www.gnu.org/copyleft/gpl.html
 * @copyright Metaways Infosystems GmbH, 2013
 * @copyright Aimeos (aimeos.org), 2014
 * @package TYPO3_Aimeos
 */


/**
 * Abstract class with common functionality for all controllers.
 *
 * @package TYPO3_Aimeos
 */
abstract class Tx_Aimeos_Controller_Abstract
	extends Tx_Extbase_MVC_Controller_ActionController
{
	private $_aimeos;
	static private $_locale;
	static private $_context;
	static private $_i18n = array();


	public function __construct()
	{
		parent::__construct();

		$this->_aimeos = Tx_Aimeos_Base::getAimeos();
	}


	/**
	 * Initializes the object before the real action is called.
	 */
	protected function initializeAction()
	{
		$context = $this->_getContext();
		$locale = $this->_getLocale( $context );

		$context->setLocale( $locale );
		$context->setCache( $this->_getCache( $context->getConfig(), $locale->getSiteId() ) );
		$context->setI18n( $this->_getI18n( array( $locale->getLanguageId() ) ) );

		$this->uriBuilder->setArgumentPrefix( 'ai' );
	}


	/**
	 * Disables Fluid views for performance reasons.
	 *
	 * return Tx_Extbase_MVC_View_ViewInterface View object
	 */
	protected function resolveView()
	{
		return null;
	}


	/**
	 * Creates the view object for the HTML client.
	 *
	 * @return MW_View_Interface View object
	 */
	protected function _createView()
	{
		$context = $this->_getContext();
		$config = $context->getConfig();

		$langid = $context->getLocale()->getLanguageId();
		$i18n = $this->_getI18n( array( $langid ) );

		// required for reloading to the current page
		$params = $this->request->getArguments();
		$params['target'] = $GLOBALS["TSFE"]->id;

		$view = new MW_View_Default();

		$helper = new MW_View_Helper_Url_Typo3( $view, $this->uriBuilder );
		$view->addHelper( 'url', $helper );

		$helper = new MW_View_Helper_Translate_Default( $view, $i18n[$langid] );
		$view->addHelper( 'translate', $helper );

		$helper = new MW_View_Helper_Parameter_Default( $view, $params );
		$view->addHelper( 'param', $helper );

		$helper = new MW_View_Helper_Config_Default( $view, $config );
		$view->addHelper( 'config', $helper );

		$sepDec = $config->get( 'client/html/common/format/seperatorDecimal', '.' );
		$sep1000 = $config->get( 'client/html/common/format/seperator1000', ' ' );
		$helper = new MW_View_Helper_Number_Default( $view, $sepDec, $sep1000 );
		$view->addHelper( 'number', $helper );

		$helper = new MW_View_Helper_FormParam_Default( $view, array( $this->uriBuilder->getArgumentPrefix() ) );
		$view->addHelper( 'formparam', $helper );

		$helper = new MW_View_Helper_Encoder_Default( $view );
		$view->addHelper( 'encoder', $helper );

		return $view;
	}


	/**
	 * Returns the cache object for the context
	 *
	 * @param \MW_Config_Interface $config Configuration object
	 * @param string $siteid Unique site ID
	 * @return \MW_Cache_Interface Cache object
	 */
	protected function _getCache( \MW_Config_Interface $config, $siteid )
	{
		$name = Tx_Aimeos_Base::getExtConfig( 'cacheName', 'Typo3' );

		switch( $name )
		{
			case 'Typo3':
				/** @todo Use modern API in TYPO3 6.3 and above */
				// \TYPO3\CMS\Core\Cache\Cache::initializeCachingFramework();
				// $cache = GeneralUtility::makeInstance( 'TYPO3\\CMS\\Core\\Cache\\CacheManager' )->getCache( 'aimeos' );

				t3lib_cache::initializeCachingFramework();

				try
				{
					$cache = $GLOBALS['typo3CacheManager']->getCache( 'aimeos' );
				}
				catch( t3lib_cache_exception_NoSuchCache $e )
				{
					$cache = $GLOBALS['typo3CacheFactory']->create( 'aimeos',
						$GLOBALS['TYPO3_CONF_VARS']['SYS']['caching']['cacheConfigurations']['aimeos']['frontend'],
						$GLOBALS['TYPO3_CONF_VARS']['SYS']['caching']['cacheConfigurations']['aimeos']['backend'],
						$GLOBALS['TYPO3_CONF_VARS']['SYS']['caching']['cacheConfigurations']['aimeos']['options']
					);
				}

				$config = array( 'siteid' => $siteid );
				break;

			default:
				$config->set( 'client/html/basket/cache/enable', false );
				$name = 'None';
				$cache = null;
				$config = array();
		}

		return MW_Cache_Factory::createManager( $name, $config, $cache );
	}


	/**
	 * Creates a new configuration object.
	 *
	 * @return MW_Config_Interface Configuration object
	 */
	protected function _getConfig()
	{
		$settings = (array) $this->settings;

		if( isset( $this->settings['typo3']['tsconfig'] ) )
		{
			$tsconfig = Tx_Aimeos_Base::parseTS( $this->settings['typo3']['tsconfig'] );
			$settings = Tx_Extbase_Utility_Arrays::arrayMergeRecursiveOverrule( $settings, $tsconfig );
		}

		return Tx_Aimeos_Base::getConfig( $settings );
	}


	/**
	 * Returns the current context.
	 *
	 * @return MShop_Context_Item_Interface Context object
	 */
	protected function _getContext()
	{
		$config = $this->_getConfig();

		if( self::$_context === null )
		{
			$context = new MShop_Context_Item_Default();

			$context->setConfig( $config );

			$dbm = new MW_DB_Manager_PDO( $config );
			$context->setDatabaseManager( $dbm );

			if( isset( $GLOBALS['TSFE']->fe_user ) ) {
				$session = new MW_Session_Typo3( $GLOBALS['TSFE']->fe_user );
			} else {
				$session = new MW_Session_None();
			}
			$context->setSession( $session );

			$logger = MAdmin_Log_Manager_Factory::createManager( $context );
			$context->setLogger( $logger );

			if( TYPO3_MODE === 'FE' && $GLOBALS['TSFE']->loginUser == 1 )
			{
				$context->setEditor( $GLOBALS['TSFE']->fe_user->user['username'] );
				$context->setUserId( $GLOBALS['TSFE']->fe_user->user[$GLOBALS['TSFE']->fe_user->userid_column] );
			}

			self::$_context = $context;
		}

		self::$_context->setConfig( $config );

		return self::$_context;
	}


	/**
	 * Creates new translation objects.
	 *
	 * @param array $langIds List of two letter ISO language IDs
	 * @return array List of translation objects implementing MW_Translation_Interface
	 */
	protected function _getI18n( array $languageIds )
	{
		$i18nPaths = Tx_Aimeos_Base::getAimeos()->getI18nPaths();

		foreach( $languageIds as $langid )
		{
			if( !isset( self::$_i18n[$langid] ) )
			{
				$i18n = new MW_Translation_Zend2( $i18nPaths, 'gettext', $langid, array( 'disableNotices' => true ) );

				if( function_exists( 'apc_store' ) === true && Tx_Aimeos_Base::getExtConfig( 'useAPC', false ) == true ) {
					$i18n = new MW_Translation_Decorator_APC( $i18n, Tx_Aimeos_Base::getExtConfig( 'apcPrefix', 't3:' ) );
				}

				if( isset( $this->settings['i18n'][$langid] ) )
				{
					$translations = Tx_Aimeos_Base::parseTranslations( (array) $this->settings['i18n'][$langid] );
					$i18n = new MW_Translation_Decorator_Memory( $i18n, $translations );
				}

				self::$_i18n[$langid] = $i18n;
			}
		}

		return self::$_i18n;
	}


	/**
	 * Returns the locale object for the context
	 *
	 * @param \MShop_Context_Item_Interface $context Context object
	 * @return \MShop_Locale_Item_Interface Locale item object
	 */
	protected function _getLocale( \MShop_Context_Item_Interface $context )
	{
		if( !isset( self::$_locale ) )
		{
			$session = $context->getSession();
			$config = $context->getConfig();


			$current = $session->get( 'aimeos/locale/sitecode', 'default' );
			$sitecode = $config->get( 'mshop/locale/site', $current );

			if( $this->request->hasArgument( 'loc-site' ) === true ) {
				$sitecode = $this->request->getArgument( 'loc-site' );
			}

			if( $sitecode !== $current ) {
				$session->set( 'aimeos/locale/sitecode', $sitecode );
			}


			$current = $session->get( 'aimeos/locale/languageid', 'en' );
			$langid = $config->get( 'mshop/locale/language', $current );

			if( isset( $GLOBALS['TSFE']->config['config']['language'] ) ) {
				$langid = $GLOBALS['TSFE']->config['config']['language'];
			}

			if( $this->request->hasArgument( 'loc-language' ) === true ) {
				$langid = $this->request->getArgument( 'loc-language' );
			}

			if( $langid !== $current ) {
				$session->set( 'aimeos/locale/languageid', $langid );
			}


			$current = $session->get( 'aimeos/locale/currencyid', 'EUR' );
			$currency = $config->get( 'mshop/locale/currency', $current );

			if( $this->request->hasArgument( 'loc-currency' ) === true ) {
				$currency = $this->request->getArgument( 'loc-currency' );
			}

			if( $currency !== $current ) {
				$session->set( 'aimeos/locale/currencyid', $currency );
			}


			$localeManager = MShop_Locale_Manager_Factory::createManager( $context );

			self::$_locale = $localeManager->bootstrap( $sitecode, $langid, $currency );
		}

		return self::$_locale;
	}


	/**
	 * Returns the output of the client and adds the header.
	 *
	 * @param Client_Html_Interface $client Html client object
	 * @return string HTML code for inserting into the HTML body
	 */
	protected function _getClientOutput( Client_Html_Interface $client )
	{
		$client->setView( $this->_createView() );
		$client->process();

		$this->response->addAdditionalHeaderData( (string) $client->getHeader() );

		return $client->getBody();
	}
}
